import { system,ItemStack,world,EquipmentSlot,EntityComponentTypes, EntityEquippableComponent, ItemDurabilityComponent, GameMode} from "@minecraft/server";
import { RawtextBuilder, showGuidePage, showPage } from "./GuideBook";
import { ModalFormData, ActionFormData } from "@minecraft/server-ui";

export const VERSION = "1.1.0";
export const VERSION_HISTORY = ["1.1.0"];

const settingsOrder = [
  "chopSpeed",
  "maxChopBlocks",
  "usesDurability",
];

const defaultValues = {
  chopSpeed: 3,
  maxChopBlocks: 16,
  usesDurability: 1,
};

  if (world.scoreboard.getObjective("settings") == undefined) {
    world.scoreboard.addObjective("settings", "settings");
  }

  settingsOrder.forEach((setting) => {
    const objective = world.scoreboard.getObjective("settings");
    try {
      objective.getScore(setting);
    } catch (error) {
      // Set the default value if an error occurs
      const defaultValue = defaultValues[setting] || 0; // Use 0 if not specified
      objective.setScore(setting, defaultValue);
    }
  });

system.runInterval(() => {
  for (const player of world.getAllPlayers()) {
    const inventory = player.getComponent(EntityComponentTypes.Inventory);

    if (inventory === undefined) return;
    const container = inventory.container;
    if (container === undefined) return;

    // Update inventory items with the typeId "sg_chop:guide"
    for (let i = 0; i < container.size; i++) {
      const itemStack = container.getItem(i);
      if (itemStack === undefined) continue;
      if (itemStack.typeId === "sg_chop:guide") {
        const lore = itemStack.getLore() || []; // Get the existing lore or start with an empty array
        if (!lore.includes("§r§7Bedrock Essential")) {
          lore.push("§r§7Bedrock Essential");
          itemStack.setLore(lore);
          container.setItem(i, itemStack);
        }
      }
    }
  }
});

function removeSingleItem(item, player) {
  const equippable = player.getComponent("equippable");
  const mainHandItem = equippable.getEquipment(EquipmentSlot.Mainhand);

  if (mainHandItem && mainHandItem.typeId === item.typeId) {
    if ((item.amount == 1)) {
      equippable.setEquipment(
        EquipmentSlot.Mainhand,
        new ItemStack("minecraft:air")
      );
    } else {
      const newItemStack = new ItemStack(item.typeId, mainHandItem.amount - 1);
      equippable.setEquipment(EquipmentSlot.Mainhand, newItemStack);
    }
  }
}
const logTypeIds = [
  "minecraft:oak_log",
  "minecraft:spruce_log",
  "minecraft:birch_log",
  "minecraft:jungle_log",
  "minecraft:acacia_log",
  "minecraft:dark_oak_log",
  "minecraft:mangrove_log",
  "minecraft:cherry_log",
  // Add any other log type IDs here
];


world.beforeEvents.playerBreakBlock.subscribe((event) => {
  let maxChopBlocks = world.scoreboard.getObjective("settings").getScore("maxChopBlocks"); // Example value, you can change this dynamically
  let blockBreakDelay = world.scoreboard.getObjective("settings").getScore("chopSpeed"); // Delay in ticks between block breaks
  const player = event.player;
  const block = event.block;
  const equippable = player.getComponent("equippable");
  const item = equippable.getEquipment(EquipmentSlot.Mainhand);
  const playerEquipment = player.getComponent(
    EntityEquippableComponent.componentId
  );
  const durabilityComponent = item.getComponent(
    ItemDurabilityComponent.componentId
  );

  // Check if the block is a log before starting tree chopping
  if (
    logTypeIds.includes(block.typeId) &&
    item &&
    item.hasTag("minecraft:is_axe")
  ) {
    // Start chopping with a delay
    system.runTimeout(
      () =>
        chopTree(
          block,
          player,
          player.dimension,
          maxChopBlocks,
          blockBreakDelay,
          item,
          durabilityComponent,
          playerEquipment
        ),
      blockBreakDelay
    );
  }
});

function chopTree(
  initialBlock,
  player,
  dimension,
  maxBlocks,
  delay,
  item,
  durabilityComponent,
  playerEquipment
) {
  let blocksToChop = [initialBlock];
  let choppedBlocks = 0;
  function chopNextBlock() {
    if (blocksToChop.length > 0 && choppedBlocks < maxBlocks && item) {
      const currentBlock = blocksToChop.shift();
      const neighboringLogs = getNeighboringLogs(currentBlock, dimension);

      neighboringLogs.forEach((logBlock) => {
        if (choppedBlocks < maxBlocks && item) {
          blocksToChop.push(logBlock);
          player.runCommand(
            `setblock ${logBlock.location.x} ${logBlock.location.y} ${logBlock.location.z} air destroy`
          );

          if (player.getGameMode() !== GameMode.creative) {
            // Check durability before incrementing
            if (
              durabilityComponent.damage < durabilityComponent.maxDurability &&
              world.scoreboard.getObjective("settings").getScore("usesDurability")
            ) {
              durabilityComponent.damage++;
            }
            if (
              durabilityComponent.damage >= durabilityComponent.maxDurability
            ) {
              player.runCommand("playsound random.break @a ~ ~ ~");
              playerEquipment.setEquipment(EquipmentSlot.Mainhand, null);
              item = null; // Set item to null to stop further chopping
              return; // Exit immediately to prevent further execution
            } else {
              playerEquipment.setEquipment(EquipmentSlot.Mainhand, item);
            }
          }

          choppedBlocks++;
        }
      });

      if (item) {
        // Only continue if the item is not broken
        system.runTimeout(() => chopNextBlock(), delay);
      }
    }
  }

  chopNextBlock(); // Start the chopping process
}

function getNeighboringLogs(block, dimension) {
  const offsets = [
    { x: 1, y: 0, z: 0 },
    { x: -1, y: 0, z: 0 },
    { x: 0, y: 1, z: 0 },
    { x: 0, y: -1, z: 0 },
    { x: 0, z: 1, y: 0 },
    { x: 0, z: -1, y: 0 },
  ];

  let neighboringLogs = [];

  offsets.forEach((offset) => {
    const neighbor = dimension.getBlock({
      x: block.location.x + offset.x,
      y: block.location.y + offset.y,
      z: block.location.z + offset.z,
    });

    if (neighbor && logTypeIds.includes(neighbor.typeId)) {
      neighboringLogs.push(neighbor);
    }
  });

  return neighboringLogs;
}
















world.afterEvents.playerSpawn.subscribe((event) => {
  const { initialSpawn, player } = event;
  if (!initialSpawn) return;

  const version = player.getDynamicProperty("sg_chop_version");
  if (version === undefined) {
    player.setDynamicProperty("sg_chop_version", VERSION);
    if (player.hasTag("sg_chop_received_guide")) {
      player.removeTag("sg_chop_received_guide");
      player.sendMessage({
        translate: "sg_chop.announce_update",
        with: [VERSION],
      });
      return;
    }
    player.dimension.spawnItem(
      new ItemStack("sg_chop:guide"),
      player.location
    );
  } else if (version !== VERSION) {
    player.setDynamicProperty("sg_chop_version", VERSION);
    player.sendMessage({
      translate: "sg_chop.announce_update",
      with: [VERSION],
    });
  }
});



const BUTTONS = [
  {
    // OPTIONS
    translation_key: "sg_chop.guide.options.title",
    icon_path: "textures/ui/book_edit_default.png",
    action: (player) => {

      const objective = world.scoreboard.getObjective("settings");

      // Create and show the modal form
      const form = new ModalFormData()
        .title("Options")
        .slider("Chop Speed", 0, 10, 1, objective.getScore("chopSpeed"))
        .slider("Max Blocks", 1, 128, 1, objective.getScore("maxChopBlocks"))
        .toggle(
          "Uses Durability",
          Boolean(objective.getScore("usesDurability"))
        );

      form.show(player).then((res) => {
        if (res.canceled) return;

        // Iterate over form values and map them to the settings labels
        const settings = res.formValues;
        settings.forEach((value, index) => {
          const label = settingsOrder[index];
          if (label) {
            objective.setScore(label, value);
          }
        });
      });
    },
  },
  {
    // CHANGELOG
    translation_key: "sg_chop.guide.changelog.title",
    icon_path: "textures/ui/book_addtextpage_default.png",
    action: (player) => {
      const body = new RawtextBuilder();
      body.pushLine({
        translate: "sg_chop.guide.changelog.body",
        with: [VERSION],
      });
      body.pushLine();
      for (const version of VERSION_HISTORY) {
        body.pushLine({
          translate: "sg_chop.guide.changelog.subtitle",
          with: [version],
        });
        body.pushLine();
        body.pushLine({
          translate: `sg_chop.guide.changelog.${version}`,
          with: ["\n"],
        });
        body.pushLine();
        body.pushLine();
      }
      showPage(player, "sg_chop.guide.changelog.title", body.build(), [
        {
          translation_key: "sg_chop.guide.common.back_button",
          action: (player) => showMainPage(player),
        },
      ]);
    },
  },
  {
    translation_key: "sg_chop.guide.common.exit_button",
  },
];

const showMainPage = (player) => {
  showGuidePage(player, "main", 4, BUTTONS);
};

world.afterEvents.itemUse.subscribe((event) => {
  system.run(() => {
    if (event.itemStack.typeId === "sg_chop:guide") {
      showMainPage(event.source);
    }
  });
});